<?php

 // THIS FILE IS PART OF WEBMATIC3 (Webmatic cube)
 // www.valarsoft.com

 defined('WM3_cEXEC') or die('Denied');

 define('WM3_cDB_SELECT',         0);
 define('WM3_cDB_CREATE_TABLE',   1);
 define('WM3_cDB_INSERT_INTO',    2);
 define('WM3_cDB_ALTER_DATABASE', 3); 

 define('WM3_cDB_INT',    0);
 define('WM3_cDB_BIGINT', 1);
 define('WM3_cDB_FLOAT',  2);
 define('WM3_cDB_DOUBLE', 3);
 define('WM3_cDB_CHAR',   4);
 define('WM3_cDB_TEXT',   5);
 define('WM3_cDB_DATE',   6);

 define('WM3_cDB_NOT_NULL',       1);
 define('WM3_cDB_AUTO_INCREMENT', 2);
 define('WM3_cDB_UNIQUE',         4);
 define('WM3_cDB_UNIQUE_KEY',     8);
 define('WM3_cDB_KEY',            16);
 define('WM3_cDB_PRIMARY_KEY',    32);
 

 require_once(WM3_cPATH_CLASSES.'DB/_mysql.php');
 //require_once(WM3_cPATH_CLASSES.'DB/_postgresql.php');//TODO

 class wm3DB
  {
   // Arrays

   // Variables
   private $db_server,$db_user,$db_password,$db_name,$db_prefix,$db_port;
   private $connection;
   private $charset;
   private $queries_affected;
   
   function __construct($db_server,$db_user,$db_password,$db_name,$db_prefix)
    {
	 $this->db_server        = $db_server;
	 $this->db_user          = $db_user;
	 $this->db_password      = $db_password;
	 $this->db_name          = $db_name;
	 $this->db_prefix        = $db_prefix;
	 $this->db_port          = 3306;//3306 = mysql, 5432 = postgresql
	 $this->queries_affected = 0;
	 $this->charset          = 'UTF8'; // Default charset
	 $this->connect();
	 $this->useCharset();
	}

   /** Connect to the database (die if fail)
    * @param void
    * @return void
    */
   public function connect()
    {
     $this->connection = @dbms_connect($this->db_server,$this->db_user,$this->db_password)
      or wm3Util::dieMessage("MySQL error","Can not connect to the MySql database server ".WM3_cCONFIG_DB_SERVER);
     dbms_select_db(WM3_cCONFIG_DB_NAME,$this->connection)
	  or wm3Util::dieMessage("MySQL error","Can not select the database ".$this->db_name);
    }

   /** Close the current database connection
    * @param void
    * @return void
    */
   public function close()
    {
	 dbms_close($this->connection);
	}

   /** Execute a query with the current connection (die if fail)
	* @param wm3DBQuery/String $query a wm3DBQuery object or a query string
	* @return Resource the query result set
    */
   public function query($query)
    {
     // Connect to the database and 
     $this->queries_affected++;
     if(is_object($query)) $query = dbms_parse_query($query,$this->charset);
     $rs = dbms_query($query,$this->connection)
	  or wm3Util::dieMessage("MySQL error","Can not execute the query:<br><b>$query</b>");
     return $rs;
    }

   public function useCharset()
    {
	 dbms_use_charset($this->charset,$this->connection);
	}

   public function fetch($result,$object = true)
    {
     // Fetch a query result (by object or array)
     if($object) return dbms_fetch_object($result);
     else        return dbms_fetch_array($result);
    }

   public function totalRows($result)
    {
     // Return the total number of rows from a query result
     return dbms_num_rows($result);
    }

   public function totalFields($result)
    {
     // Return the total number of fields from a query result
     return dbms_num_fields($result);
    }

   public function listTables()
    {
     // Return the database tables list
     return dbms_list_tables($this->db_name,$this->connection);
    }

   public function listFields($table)
    {
     // Return fields list for a table
     return dbms_list_fields($this->db_name,$table,$this->connection);
    }

   public function tableName($result,$i)
    {
     // Return table name with index $i
     return dbms_tablename($result,$i);
    }

   public function fieldName($result,$i)
    {
     // Return field name with index $i
     return dbms_field_name($result,$i);
    }

   public function fieldType($result,$i)
    {
     // Return field type with index $i
     return dbms_field_type($result,$i);
    }

   public function fieldLenght($result,$i)
    {
     // Return the lenght of field with index $i
     return dbms_field_len($result,$i);
    }

   public function fieldFlags($result,$i)
    {
     // Return the flags of field with index $i
     return dbms_field_flags($result,$i);
    }

   public function saveAllTableFields($tb,$_F,$_R = array(),$id = true)
    {
     // Insert all fields in a table filled with values ($_F its an array with fields/values)
     // The field in the $_R array (remove array) will not performed
	 // if $id its true the first field in $_F will be processed as an autoincrement key
     $n = 0;
     $part1 = $part2 = "";
     while(list($k,$v) = each($_F))
      {
       $use = 1;
       for($j = 0; $j < sizeof($_R); $j++) if($k == $_R[$j]) $use = 0;
       if($use)
        {
         if($n == 0) {
			          if($id) { $part1 .= $k; $part2 .= "''";       }
					  else    { $part1 .= $k; $part2 .= "'".$v."'"; }
					 }
         else        { $part1 .= ", ".$k; $part2 .= ", '".$v."'"; }
         $n++;
        }
      }
     $query = "INSERT INTO ".WM3_cCONFIG_DB_PREFIX."$tb (".$part1.") VALUES (".$part2.")";
     $this->query($query);
    }

   public function countTable($table,$where = '')
    {
     $row = $this->fetch($this->query("SELECT COUNT(*) AS total FROM $table $where"));
     return $row->total;
    }

   public function resultToArray($result,$object = true)
    {
	 // Convert a resultset to an array (of objcts if $objects is true)
	 $_ARRAY= array();
     while($row = $this->fetch($result,$object))
      {
       $_ARRAY[] = $row;
      }
     return $_ARRAY;
    }

   public function getFirstRowField($table,$field)
    {
	 $rs = $this->query("SELECT $field FROM $table LIMIT 0,1");
     if($this->totalRows($rs) == 1)
	  {
	   $row = $this->fetch($rs,false);
	   return $row[$field];
	  }
	 return '';
    }

   public function getLastInsertID()
    {
     $row = $this->fetch($this->query("SELECT LAST_INSERT_ID()"),false);
     return $row[0];
    }

   public function getNextAutoIncrementID($table)
    {
     $row = $this->fetch($this->query("SHOW TABLE STATUS LIKE '$table'"),false);
     return $row['Auto_increment'];
    }

   public function getRSPosition($rs,$field,$id_value)
    {
	 $i = 0;
	 while($row = $this->fetch($rs,false))
	  {
	   if($row[$field] == $id_value) return $i;
	   $i++;
	  }
     return;
    }

   public function getCharset() { return $this->charset; }

   public function existValue($table,$field,$value,$id_name = '',$id_value = 0)
    {
	 $query = "SELECT $field FROM $table WHERE $field = '$value'";
	 if($id_name) $query .= " AND $id_name <> '$id_value'";
	 $rs = $this->query($query);
     if($this->totalRows($rs) != 0) return true;
	 return false;
    }

   public function countDBTables()
    {
	 // Return the number of total tables in the database
	 $rs = $this->query("SHOW TABLES LIKE '".WM3_cCONFIG_DB_PREFIX."%'");
	 return $this->totalRows($rs);
	}

   public function countDBUsedSpace()
    {
	 // Return the number of total bytes used by database
	 $bytes = 0;
	 $rs = $this->query("SHOW TABLE STATUS LIKE '".WM3_cCONFIG_DB_PREFIX."_%'");
     while($row = $this->fetch($rs,false))
      {
	   $bytes += $row['Data_length'] + $row['Index_length'];
      }
     return $bytes;
	}

   public function existsTable($table)
    {
	 return dbms_exists_table($table,$this->connection);
	}

   public function getDBVersion()
    {
	 if($this->existsTable(WM3_cCONFIG_DB_PREFIX."_wcm"))
	  {
       $rs = $this->query("SELECT * FROM ".WM3_cCONFIG_DB_PREFIX."_wcm");
       if($this->totalRows($rs) == 1)
	    {
	     $row = $this->fetch($rs);
         return $row->wcm;
	    }
	   else return 1.0; // Database version 1.0
      }
	 if($this->existsTable(WM3_cCONFIG_DB_PREFIX."_database"))
	  {
       $rs = $this->query("SELECT * FROM ".WM3_cCONFIG_DB_PREFIX."_database");
       $row = $this->fetch($rs);
       return $row->database_version;
      }
	 return 0; // Database not installed or incorrect installation
    }

   public static function checkConnection($server,$user,$password,$name)
    {
     $connection = dbms_connect($server,$user,$password);
     return dbms_select_db($name,$connection);
    }

   public static function checkDatabaseServerVersion($server,$user,$password)
    {
     $connection = dbms_connect($server,$user,$password);
     if(!$connection) return 0;
	 if(!(floatval(dbms_get_server_info($connection)) >= 4.1)) return 0;
     else return 1;
    }

   public static function serverInformations($server,$user,$password)
    {
     // Return the database server informations
     $connection = dbms_connect($server,$user,$password);
     return dbms_get_server_info($connection);
    }

   public static function existsDatabase()
    {
     // Check if the database and tables exists
     $connection = dbms_connect(WM3_cCONFIG_DB_SERVER,WM3_cCONFIG_DB_USER,WM3_cCONFIG_DB_PASSWORD)
      or wm3Util::dieMessage("MySQL error","Can not connect to the MySql database server ".WM3_cCONFIG_DB_SERVER);
     dbms_select_db(WM3_cCONFIG_DB_NAME,$connection)
	  or wm3Util::dieMessage("MySQL error","Can not select the database ".WM3_cCONFIG_DB_NAME);
	 //return dbms_query("SELECT * FROM ".WM3_cCONFIG_DB_PREFIX."_wcm",$connection);//TODO verificare bene
	 return dbms_query("SELECT * FROM ".WM3_cCONFIG_DB_PREFIX."_wcm",$connection) || 
	        dbms_query("SELECT * FROM ".WM3_cCONFIG_DB_PREFIX."_database",$connection);
    }

   public function getConnection() { return $this->connection; }

   public function getQueriesAffected() { return $this->queries_affected; }

  }

?>
